(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 4.0,
MathReader 4.0, or any compatible application. The data for the notebook 
starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     16754,        529]*)
(*NotebookOutlinePosition[     17691,        559]*)
(*  CellTagsIndexPosition[     17647,        555]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["qh\[Hyphen]math", "Title"],

Cell[TextData[{
  StyleBox["MathLink",
    FontSlant->"Italic"],
  " Interface to QHull's Delaunay Triangulation Program"
}], "Subtitle"],

Cell[TextData[{
  "Original work by ",
  ButtonBox["Alban Tsui",
    ButtonData:>{
      URL[ "mailto:a.tsui@ic.ac.uk"], None},
    ButtonStyle->"Hyperlink"],
  "\nRevised and documented by ",
  ButtonBox["P.J. Hinton",
    ButtonData:>{
      URL[ "mailto:paulh@wolfram.com"], None},
    ButtonStyle->"Hyperlink"]
}], "Author"],

Cell[TextData[{
  "The qh\[Hyphen]math package allows ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " users to access the QHull Delaunay triangulation algorithm directly from \
the kernel.  It consists of C\[Hyphen]language source code, a ",
  StyleBox["MathLink",
    FontSlant->"Italic"],
  " template file, and several support files for compilation with development \
tools on UNIX\[RegisteredTrademark], Linux\[RegisteredTrademark], MacOS\
\[RegisteredTrademark], and Microsoft Windows \[RegisteredTrademark]. "
}], "Abstract"],

Cell["\<\
Use of this software implies consent to terms and conditions of the \
license agreement in the file COPYING that is included with the \
qh\[Hyphen]math distribution.\
\>", "Abstract"],

Cell[TextData[{
  "This software package is not an official product of Wolfram Research, Inc. \
 It is provided as a service to the ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " user community as an ongoing freelance project."
}], "Abstract"],

Cell["\<\
THE AUTHORS DISCLAIM ALL WARRANTIES, EXPRESS OR IMPLIED, WITH \
REGARD TO THIS SOFTWARE, INCLUDING WITHOUT LIMITATION ALL IMPLIED WARRANTIES \
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, AND IN NO EVENT \
SHALL THE AUTHORS BE LIABLE FOR ANY SPECIAL, INDIRECT OR 
CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, \
DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, TORT (INCLUDING \
NEGLIGENCE) OR STRICT LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE \
OR PERFORMANCE OF THIS SOFTWARE.\
\>", "Abstract"],

Cell[TextData[{
  "Please do not contact the authors for copies of precompiled binaries of \
this software.  The ",
  StyleBox["MathLink",
    FontSlant->"Italic"],
  " license does not permit royalty\[Hyphen]free redistribution of derivative \
works.  Aside from the files included with this package, no assistance in \
configuring a build environment for this software will be provided by the \
authors."
}], "Abstract"],

Cell[CellGroupData[{

Cell["Requisite Materials", "Section 1"],

Cell[TextData[{
  "This package is not self\[Hyphen]contained.  You will need to obtain the \
source code to Qhull version 2.6, which may be ",
  ButtonBox["downloaded from its official website",
    ButtonData:>{
      URL[ "http://www.geom.umn.edu/software/download/qhull.html"], None},
    Active->True,
    ButtonStyle->"Hyperlink"],
  ". You will also need appropriate development tools for the platform on \
which you will be running qh-math.  Your development tool should provide an \
ANSI C compiler and a linker for creating libraries.  A program maintenance \
utility \[Dash] such as make \[Dash] will also be helpful.  The table below \
lists the tools on which qh\[Hyphen]math has been compiled successfully."
}], "Text"],

Cell[TextData[Cell[BoxData[
    FormBox[GridBox[{
          {"Platform", 
            RowBox[{"Development", " ", "Tool"}]},
          {"Linux", 
            RowBox[{"GCC", "-", 
              RowBox[{"egcs", " ", "2.91", 
                RowBox[{".66", "/", "GNU"}], " ", "make"}]}]},
          {"Linux", 
            RowBox[{
            "Metrowerks", " ", "Code", " ", "Warrior", " ", "3.3", ".5", "  ",
               "GNU", " ", "Edition"}]},
          {"Windows", 
            RowBox[{"Borland", " ", 
              RowBox[{"C", "++"}], " ", 
              RowBox[{"5.5", "/", "MAKE"}]}]},
          {"Windows", 
            RowBox[{"Microsoft", "  ", 
              RowBox[{"C", "/", 
                RowBox[{"C", "++"}]}], " ", 
              RowBox[{"12.0", "/", "NMAKE"}]}]},
          {"MacOS", 
            RowBox[{"MetroWerks", " ", "CodeWarrior", " ", "4.0"}]}
          }], TextForm]],
  GridBoxOptions->{ColumnLines->True}]], "Text",
  GridBoxOptions->{ColumnAlignments->{Left},
  RowLines->{True, False}}],

Cell[TextData[{
  "Finally, you will need to obtain a copy of the latest version of the ",
  StyleBox["MathLink",
    FontSlant->"Italic"],
  " Developer's Kit for your target platform.  The kit is included on all \
professional versions of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ".  Kits may also be downloaded from the Wolfram Research ",
  ButtonBox["technical support website",
    ButtonData:>{
      URL[ "http://support.wolfram.com/MathLink/Updates/MLDK.html"], None},
    ButtonStyle->"Hyperlink"],
  "."
}], "Text"],

Cell[TextData[{
  "Note that Windows users ",
  StyleBox["must", "TI"],
  " use a compiler that is capable of linking against dynamic link libraries \
(DLLs) because ",
  StyleBox["MathLink",
    FontSlant->"Italic"],
  " is implemented as a shared library on that platform.  You cannot use a \
pure MS\[Hyphen]DOS based compiler, but you may use command line versions of \
Win32 compilers."
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Building the Program", "Section"],

Cell["\<\
It is assumed that you are familiar with your choice of development \
tools.  If you are not comfortable with invoking your compiler and linker \
through your tool's program maintenance utility (e.g. make or IDE project \
settings), please review the tool's documentation before proceeding \
further.\
\>", "Text"],

Cell["\<\
Building the qh\[Hyphen]math executable consists of performing the \
following steps.\
\>", "Text"],

Cell["\<\
1) Compiling selected Qhull source files to create a static \
library.  \
\>", "Text"],

Cell[TextData[{
  "For UNIX and Linux users, the Qhull distribution includes a target for \
building the library (",
  StyleBox["libqhull.a", "MR"],
  ") in its own ",
  StyleBox["Makefile", "MR"],
  ".  "
}], "Text"],

Cell[TextData[{
  "The files ",
  StyleBox["qh-lib.BCC", "MR"],
  " and ",
  StyleBox["qh-lib.MS", "MR"],
  " included with qh\[Hyphen]math are written for  the Borland ",
  StyleBox["MAKE", "MR"],
  " and Microsoft ",
  StyleBox["NMAKE", "MR"],
  " program maintenance utilities respectively.  Simply drop these files into \
the Qhull source directory, edit the files to reflect paths on your local \
system, and run the appropriate program maintenance utility. "
}], "Commentary"],

Cell["\<\
Under MacOS, no separate library is built.  Instead, the required \
object files are linked directly into the final executable.  See the \
CodeWarrior for Macintosh notes later on in this section.\
\>", "Commentary"],

Cell[TextData[{
  "2) Creating a C\[Hyphen]language source file from the ",
  StyleBox["MathLink",
    FontSlant->"Italic"],
  " template file ",
  StyleBox["qh-math.tm", "MR"],
  " with the aid of ",
  StyleBox["mprep", "MR"],
  " (UNIX and Windows) or ",
  StyleBox["SAmprep", "MR"],
  " (MacOS)."
}], "Text"],

Cell[TextData[{
  "3) Compiling the C\[Hyphen]language source files ",
  StyleBox["qh-math.c", "MR"],
  " and the file generated in step (2) to object code."
}], "Text"],

Cell[TextData[{
  "4) Linking the object code in (3) with the library created in (1) and the \
",
  StyleBox["MathLink",
    FontSlant->"Italic"],
  " library."
}], "Text"],

Cell["\<\
For your convenience, support files to accomplish steps (2) through \
(4) have been included with this distribution.  \
\>", "Commentary"],

Cell[TextData[{
  "The files ",
  StyleBox["Makefile.GNU", "MR"],
  ", ",
  StyleBox["Makefile.BCC", "MR"],
  ", and ",
  StyleBox["Makefile.MS", "MR"],
  " are written for GNU ",
  StyleBox["make", "MR"],
  ", Borland ",
  StyleBox["MAKE", "MR"],
  ", and Microsoft ",
  StyleBox["NMAKE", "MR"],
  " respectively.  The GNU makefile should work on UNIX andLinux, provided \
that GNU ",
  StyleBox["make", "MR"],
  " has been installed on your system.  The makefile may also work with other \
UNIX ",
  StyleBox["make", "MR"],
  " variants.  You ",
  StyleBox["will", "TI"],
  " need to adjust the path settings in the makefiles to reflect the location \
of the ",
  StyleBox["MathLink",
    FontSlant->"Italic"],
  " Developer's Kit and the Qhull library on your system."
}], "Commentary"],

Cell[TextData[{
  "The file ",
  StyleBox["qh-math.mcp", "MR"],
  " is a Metrowerks CodeWarrior for Linux project file.  To use this project \
file, you will need to adjust the path settings in ",
  StyleBox["mprep.psh", "MR"],
  " to point to the location of the ",
  StyleBox["MathLink",
    FontSlant->"Italic"],
  " Developer's kit on your system.  This shell script is invoked by the \
CodeWarrior Shell Tool to process the ",
  StyleBox["MathLink",
    FontSlant->"Italic"],
  " template file.  You will need to add the path to the ",
  StyleBox["MathLink",
    FontSlant->"Italic"],
  " header file (",
  StyleBox["mathlink.h", "MR"],
  ") in the User Paths of the Access Paths project setting.  You will also \
need to change the Linker Flags on the GNU Linker so that the directories \
containing the Qhull library and the ",
  StyleBox["MathLink",
    FontSlant->"Italic"],
  " library (",
  StyleBox["libML.a", "MR"],
  ") are included."
}], "Commentary"],

Cell[TextData[{
  "The file ",
  StyleBox["qh-math.hqx", "MR"],
  " is a BinHex encoded MetroWerks CodeWarrior project for the Macintosh \
platform.  In order for the project to build, you will need to add the \
following source code files from the Qhull distribution to the project file \
list."
}], "Commentary"],

Cell[TextData[Cell[BoxData[
    FormBox[GridBox[{
          {\(geom . c\), \(geom2 . c\), \(global . c\)},
          {\(io . c\), \(mem . c\), \(merge . c\)},
          {\(poly . c\), \(poly2 . c\), \(qhull . c\)},
          {\(qset . c\), \(stat . c\), \(user . c\)}
          }], TextForm]],
  GridBoxOptions->{ColumnAlignments->{Left}}]], "Commentary"],

Cell[TextData[{
  "You will also need to add to the project listing the ",
  StyleBox["MathLink",
    FontSlant->"Italic"],
  "  Stubs file that is provided with the ",
  StyleBox["MathLink",
    FontSlant->"Italic"],
  " Developer's Kit."
}], "Commentary"],

Cell[TextData[{
  "The project does not include a step for processing the ",
  StyleBox["MathLink",
    FontSlant->"Italic"],
  " template file, so you must run ",
  StyleBox["SAmprep", "MR"],
  " outside of CodeWarrior.  Information on how to do this may be found in \
the DeveloperGuide.nb notebook that accompanies the Macintosh version of the \
",
  StyleBox["MathLink",
    FontSlant->"Italic"],
  "  Developer's Kit."
}], "Commentary"],

Cell["\<\
Building the project on the Macintosh may cause the CodeWarrior \
compiler to issue several hundred warning messages (symbolized by a black \
exclamation point within a yellow triangle.  These messages are benign and \
may be disregarded.\
\>", "Commentary"],

Cell[TextData[{
  "Once the executable is built, you may place it anywhere in the kernel's \
file search path list (",
  StyleBox["$Path", "MR"],
  ")"
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Using the Program", "Section"],

Cell[TextData[{
  "To access the function in the executable, use the ",
  StyleBox["Install", "MR"],
  " function."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"lnk", " ", "=", " ", 
      RowBox[{"Install", "[", "\"\<qh-math.exe\>\"", "]"}]}]], "Input"],

Cell[BoxData[
    RowBox[{"LinkObject", "[", 
      RowBox[{"\<\"'/home/paulh/.Mathematica/4.0/AddOns/Applications/qh-math.\
exe'\"\>", ",", "2", ",", "2"}], "]"}]], "Output"]
}, Open  ]],

Cell[TextData[{
  "The executable provides a single function named ",
  StyleBox["qDelaunayTriangulation", "MR"],
  "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"LinkPatterns", "[", "lnk", "]"}]], "Input"],

Cell[BoxData[
    RowBox[{"{", 
      TagBox[
        RowBox[{"qDelaunayTriangulation", "[", 
          RowBox[{
            RowBox[{"QHull`Private`y", ":", 
              RowBox[{"{", "__List", "}"}]}], ",", 
            RowBox[{"QHull`Private`opts_String", ":", "\<\"\"\>"}]}], "]"}],
        HoldForm], "}"}]], "Output"]
}, Open  ]],

Cell["\<\
A usage message provides an example of the function's syntax.\
\>", \
"Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"?", "qDelaunayTriangulation"}]], "Input"],

Cell[BoxData["\<\"qDelaunayTriangulation[{{x1, y1}, {x2, y2}, ...}] computes \
the Delaunay triangulation of a set of points using the qhull library.\"\>"], \
"Print"]
}, Open  ]],

Cell["\<\
This function computes the triangulation for a set of random \
points.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{"pts", " ", "=", " ", 
        RowBox[{"Table", "[", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"Random", "[", "]"}], ",", 
                RowBox[{"Random", "[", "]"}]}], "}"}], ",", 
            RowBox[{"{", "10", "}"}]}], "]"}]}], ";"}], "\[IndentingNewLine]",
   
    RowBox[{"qDelaunayTriangulation", "[", "pts", "]"}]}], "Input"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"8", ",", "1", ",", "10"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"5", ",", "9", ",", "10"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"2", ",", "8", ",", "1"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"2", ",", "1", ",", "7"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"2", ",", "4", ",", "7"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"2", ",", "8", ",", "9"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"2", ",", "4", ",", "9"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"3", ",", "8", ",", "9"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"3", ",", "5", ",", "9"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"6", ",", "3", ",", "8"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"6", ",", "3", ",", "5"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"6", ",", "8", ",", "10"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"6", ",", "5", ",", "10"}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
  "You may also specify Qhull\[Hyphen]native command line options in the form \
of a string as the second argument to ",
  StyleBox["qDelaunayTriangulation", "MR"],
  ".  Refer to the Qhull documentation for a complete description of these \
options."
}], "Text"],

Cell[TextData[{
  "When the function is no longer needed, you may close the external program \
with ",
  StyleBox["Uninstall", "MR"],
  "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Uninstall", "[", "lnk", "]"}]], "Input"],

Cell[BoxData["\<\"'/home/paulh/.Mathematica/4.0/AddOns/Applications/qh-math.\
exe'\"\>"], "Output"]
}, Open  ]]
}, Closed]]
}, Open  ]]
},
FrontEndVersion->"4.0 for Macintosh",
ScreenRectangle->{{0, 1024}, {0, 748}},
ScreenStyleEnvironment->"Brackets",
WindowSize->{520, 600},
WindowMargins->{{190, Automatic}, {Automatic, 33}},
ShowCellLabel->False,
StyleDefinitions -> "Report.nb",
MacintoshSystemPageSetup->"\<\
00<0001804P000000]P2:?oQon82n@960dL5:0?l0080001804P000000]P2:001
0000I00000400`<300000BL?00400@0000000000000006P801T1T00000000000
00000000000000000000000000000000\>"
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1739, 51, 32, 0, 74, "Title"],
Cell[1774, 53, 137, 4, 65, "Subtitle"],
Cell[1914, 59, 328, 11, 82, "Author"],
Cell[2245, 72, 542, 11, 98, "Abstract"],
Cell[2790, 85, 193, 4, 70, "Abstract"],
Cell[2986, 91, 255, 6, 72, "Abstract"],
Cell[3244, 99, 563, 9, 178, "Abstract"],
Cell[3810, 110, 422, 9, 96, "Abstract"],

Cell[CellGroupData[{
Cell[4257, 123, 40, 0, 70, "Section 1"],
Cell[4300, 125, 733, 13, 124, "Text"],
Cell[5036, 140, 1022, 26, 115, "Text"],
Cell[6061, 168, 542, 14, 76, "Text"],
Cell[6606, 184, 403, 10, 76, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[7046, 199, 39, 0, 41, "Section"],
Cell[7088, 201, 324, 6, 68, "Text"],
Cell[7415, 209, 109, 3, 26, "Text"],
Cell[7527, 214, 96, 3, 26, "Text"],
Cell[7626, 219, 217, 7, 41, "Text"],
Cell[7846, 228, 482, 12, 60, "Commentary"],
Cell[8331, 242, 226, 4, 34, "Commentary"],
Cell[8560, 248, 311, 11, 43, "Text"],
Cell[8874, 261, 169, 4, 41, "Text"],
Cell[9046, 267, 172, 6, 42, "Text"],
Cell[9221, 275, 148, 3, 34, "Commentary"],
Cell[9372, 280, 789, 26, 73, "Commentary"],
Cell[10164, 308, 966, 26, 99, "Commentary"],
Cell[11133, 336, 314, 7, 47, "Commentary"],
Cell[11450, 345, 355, 7, 63, "Commentary"],
Cell[11808, 354, 257, 8, 34, "Commentary"],
Cell[12068, 364, 441, 12, 60, "Commentary"],
Cell[12512, 378, 268, 5, 47, "Commentary"],
Cell[12783, 385, 163, 5, 41, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[12983, 395, 36, 0, 41, "Section"],
Cell[13022, 397, 127, 4, 28, "Text"],

Cell[CellGroupData[{
Cell[13174, 405, 120, 2, 32, "Input"],
Cell[13297, 409, 175, 3, 63, "Output"]
}, Open  ]],
Cell[13487, 415, 131, 4, 28, "Text"],

Cell[CellGroupData[{
Cell[13643, 423, 70, 1, 32, "Input"],
Cell[13716, 426, 323, 8, 63, "Output"]
}, Open  ]],
Cell[14054, 437, 87, 3, 28, "Text"],

Cell[CellGroupData[{
Cell[14166, 444, 68, 1, 32, "Input"],
Cell[14237, 447, 167, 2, 70, "Print"]
}, Open  ]],
Cell[14419, 452, 94, 3, 28, "Text"],

Cell[CellGroupData[{
Cell[14538, 459, 427, 11, 48, "Input"],
Cell[14968, 472, 1104, 28, 79, "Output"]
}, Open  ]],
Cell[16087, 503, 279, 6, 60, "Text"],
Cell[16369, 511, 151, 5, 44, "Text"],

Cell[CellGroupData[{
Cell[16545, 520, 67, 1, 32, "Input"],
Cell[16615, 523, 99, 1, 63, "Output"]
}, Open  ]]
}, Closed]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

